@@ -9,7 +9,7 @@ rTom  <- mxModel( "Tom", aMat, aCon)
 summary(rTomRun <- mxRun(rTom))
 
 rTomRun$aMat$values[1,1] <- .1
-omxCheckWarning(logLik(rTomRun), "MxModel 'Tom' was modified since it was run. The log likelihood may be out-of-date.")
+omxCheckWarning(logLik(rTomRun), "MxModel 'Tom' was modified since it was run.")
 
 rNeal  <- mxRename(rTomRun, newname="Neal")
 summary(rNeal)
