@@ -302,4 +302,29 @@ omxCheckCloseEnough(covs.sum$informationCriteria['BIC:','par'], 242.40, .01)
 
 omxCheckCloseEnough(mg.sum$informationCriteria[c(1:4,6)], c(-6936.958, -28889.347, 2063.042, 2195.552, 2109.798), .01)
 
+# -----------------------------------
+
+data(demoOneFactor)
+manifests <- names(demoOneFactor)
+latents <- c("G1")
+fit1 <- mxRun(mxModel(model="One Factor", type="RAM",
+                      manifestVars = manifests, latentVars = latents,
+                      mxPath(from = latents, to=manifests),
+                      mxPath(from = manifests, arrows = 2),
+                      mxPath(from = latents, arrows = 2, free = FALSE, values = 1.0),
+                      mxData(cov(demoOneFactor), type = "cov", numObs = 500)
+))
+
+latents <- c("G1", "G2")
+fit2 <- mxModel(model="Two Factor", type="RAM",
+                manifestVars = manifests, latentVars = latents,
+                mxPath(from = latents[1], to=manifests[1:3]),
+                mxPath(from = latents[2], to=manifests[4:5]),
+                mxPath(from = manifests, arrows = 2),
+                mxPath(from = latents, arrows = 2, free = FALSE, values = 1.0),
+                mxData(cov(demoOneFactor), type = "cov", numObs=500)
+)
+
+omxCheckError(mxCompare(fit1, fit2, boot=T),
+              "Cannot bootstrap null model 'Two Factor' ; Does this model contain raw data?")
 
