@@ -130,9 +130,16 @@ i3 <- mxModel(m2,
 				     mxComputeReportDeriv())))
 i3 <- mxRun(i3, silent=TRUE)
 omxCheckTrue(i3$output$infoDefinite)
-omxCheckCloseEnough(log(i3$output$conditionNumber), 2.9, 1)
-omxCheckCloseEnough(log(det(i3$output$hessian)), 322, 40)
+omxCheckCloseEnough(log(i3$output$conditionNumber), 2.9, .1)
+omxCheckCloseEnough(log(det(i3$output$hessian)), 282.36, .1)
 #cat(deparse(round(i3$output$standardErrors,3)))
+ose <- c(0.144, 0.109, 0.11, 0.243, 0.15, 0.119, 0.135, 0.105,
+         0.106, 0.093, 0.172, 0.162, 0.154, 0.126, 0.131, 0.147, 0.12,
+         0.113, 0.201, 0.102, 0.106, 0.142, 0.118, 0.115, 0.121, 0.178,
+         0.131, 0.129, 0.129, 0.138, 0.142, 0.126, 0.145, 0.115, 0.113,
+         0.116, 0.116, 0.148, 0.158, 0.124, 0.112, 0.097, 0.102, 0.128,
+         0.097, 0.098, 0.103, 0.106, 0.111, 0.133, 0.155, 0.11)
+omxCheckCloseEnough(c(i3$output$standardErrors), ose, .001)
 
 refModels <- mxRefModels(m2, run=TRUE)
 
