@@ -56,4 +56,5 @@ omxCheckEquals(length(omxGetParameters(testm)), 1L)
 
 vn <- omxCheckWarning(mxModel("varName", type="RAM",
                               manifestVars=':-<', latentVars = ":-)"),
-                      "The name ':-<' is illegal because it contains the characters '-', ':', and '<' in FUN(X[[i]], ...)")
+                      c("The name ':-)' is illegal because it contains the characters '-' and ':' in FUN(X[[i]], ...)",
+                        "The name ':-<' is illegal because it contains the characters '-', ':', and '<' in FUN(X[[i]], ...)"))
